/**************************************************************************//**
 * @file
 * @brief LCD Demo for EFM32_G8xx_DK, Energy Mode 0 (polled)
 * @author Energy Micro AS
 * @version 1.1.1
 ******************************************************************************
 * @section License
 * <b>(C) Copyright 2010 Energy Micro AS, http://www.energymicro.com</b>
 ******************************************************************************
 *
 * This source code is the property of Energy Micro AS. The source and compiled
 * code may only be used on Energy Micro "EFM32" microcontrollers.
 *
 * This copyright notice may not be removed from the source code nor changed.
 *
 * DISCLAIMER OF WARRANTY/LIMITATION OF REMEDIES: Energy Micro AS has no
 * obligation to support this Software. Energy Micro AS is providing the
 * Software "AS IS", with no express or implied warranties of any kind,
 * including, but not limited to, any implied warranties of merchantability
 * or fitness for any particular purpose or warranties against infringement
 * of any proprietary rights of a third party.
 *
 * Energy Micro AS will not be liable for any consequential, incidental, or
 * special damages, or any other relief, or for any claim by any third party,
 * arising from your use of this Software.
 *
 *****************************************************************************/
#include <stddef.h>
#include <stdbool.h>
#include "chip.h"
#include "efm32.h"
#include "rtc.h"
#include <stdio.h>
#include "bsp.h"

volatile uint32_t msTicks; /* counts 1ms timeTicks */

void AdditionalConfig(void);
void ClockUpdate(void);
void Board_Test(void);
void KillDevice(void);

/**************************************************************************//**
 * @brief SysTick_Handler
 * Interrupt Service Routine for system tick counter
 *****************************************************************************/
void SysTick_Handler(void)
{
  msTicks++;       /* increment counter necessary in Delay()*/
}

/**************************************************************************//**
 * @brief Delays number of msTick Systicks (typically 1 ms)
 * @param dlyTicks Number of ticks to delay
 *****************************************************************************/
void Delay(uint32_t dlyTicks)
{
  uint32_t curTicks;

  curTicks = msTicks;
  while ((msTicks - curTicks) < dlyTicks) ;
}

/**************************************************************************//**
 * @brief  Main function
 *****************************************************************************/
int main(void)
{
	/* Chip errata */
	CHIP_init();
	
	// turn on clocking of all the modules
	// *NOTE: THIS IS A COMPATABILITY FIX, previous releases had all the modules clocked at power on
	CMU_TypeDef *cmu = CMU;
	cmu->HFCORECLKEN0 |= 0x0F;
	cmu->HFPERCLKEN0 |= 0xFFFF;
	
	// turn on clocking of all the modules
	// *NOTE: THIS IS A COMPATABILITY FIX, previous releases had all the modules clocked at power on
	cmu->HFCORECLKEN0 |= 0x0F;
	cmu->HFPERCLKEN0 |= 0xFFFF;
	
	// configure oscillator
	// enable LFX0 oscillator
	cmu->OSCENCMD     |= CMU_OSCENCMD_LFXOEN;
	while (!(cmu->STATUS & CMU_STATUS_LFXORDY)) ;

	// select HFX0 (@16MHz)
	cmu->OSCENCMD     |= CMU_OSCENCMD_HFXOEN;
	while (!(cmu->STATUS & CMU_STATUS_HFXORDY)) ;
	cmu->CMD |= CMU_CMD_HFCLKSEL_HFXO;
	
	cmu->HFCORECLKDIV = CMU_HFCORECLKDIV_HFCORECLKDIV_HFCLK4;
	cmu->HFPERCLKDIV =	CMU_HFPERCLKDIV_HFPERCLKDIV_HFCLK4 |
					CMU_HFPERCLKDIV_HFPERCLKEN;

	
	/* Ensure core frequency has been updated */
	SystemCoreClockUpdate();
	/* Setup SysTick Timer for 1 msec interrupts  */
	if (SysTick_Config(SystemCoreClock / 1000)) while (1) ;
	

	AdditionalConfig();
	
	/* Initialize necessary clocks and controller */
	RTC_init();
	RTC_trigger(500, ClockUpdate);
	AdditionalConfig();
	
	/* Run demo */
	Board_Test();
}

void AdditionalConfig(void)
{
	CMU_TypeDef *cmu = CMU;
	GPIO_TypeDef *gpio = GPIO;
	ADC_TypeDef *adc = ADC0;
	
	/* Configure button PA1, as input with pull up */
	cmu->HFPERCLKEN0 |= CMU_HFPERCLKEN0_GPIO; // turn on clock
	gpio->P[0].MODEL &= ~(_GPIO_P_MODEL_MODE1_MASK);
	gpio->P[0].MODEL |= GPIO_P_MODEL_MODE1_INPUTPULLFILTER;
	gpio->P[0].DOUT |= (1 << 1);
	
	/* enable LEDs PA0*/
	gpio->P[0].MODEL &= ~(_GPIO_P_MODEL_MODE0_MASK);
	gpio->P[0].MODEL |= GPIO_P_MODEL_MODE0_PUSHPULLDRIVE;
	gpio->P[0].DOUT &= ~(1 << 0);

	gpio->P[0].CTRL = 0x2; // 20mA outputs

	/* configure the ADC module to sample the battery level */
	cmu->HFPERCLKEN0 |= CMU_HFPERCLKEN0_ADC0; // turn on clock

	adc->SINGLECTRL = 0x0;
	adc->SINGLECTRL |= ADC_SINGLECTRL_SINGLEREF_1V25; // reference is 1.25V internal
	adc->SINGLECTRL |= ADC_SINGLECTRL_SINGLEAT_8CYCLES; // aquisition time is 8 cycles
	adc->SINGLECTRL |= ADC_SINGLECTRL_SINGLESEL_CH4; // battery resistor divider is connected to CH4
	adc->IEN = ADC_IF_SINGLE;
	NVIC_ClearPendingIRQ(ADC0_IRQn);
	NVIC_EnableIRQ(ADC0_IRQn);
	
	// wait for warmup to finish
	while(adc->STATUS & ADC_STATUS_WARM);	

	
	// turn off unnecessary modules
	cmu->HFCORECLKEN0 &= ~(	CMU_HFCORECLKEN0_AES|
						CMU_HFCORECLKEN0_DMA|
						CMU_HFCORECLKEN0_EBI );
	
	cmu->HFPERCLKEN0 &= ~(	CMU_HFPERCLKEN0_USART0|
						CMU_HFPERCLKEN0_USART1|
						CMU_HFPERCLKEN0_USART2|
						CMU_HFPERCLKEN0_UART0|
						CMU_HFPERCLKEN0_ACMP0|
						CMU_HFPERCLKEN0_ACMP1|
						CMU_HFPERCLKEN0_PRS|
						CMU_HFPERCLKEN0_DAC0|
						CMU_HFPERCLKEN0_I2C0 );
}

